﻿Public Class IPData

   ''' <summary>Länge der SubnetzMaske</summary>
   Public NetMaskLength As Int32 = Nothing
   Public Address As Int32 = Nothing

   ''' <summary>SubnetzMaske in segmentierter Darstellung</summary>
   Public ReadOnly Property NetBitMask As String
      Get
         Return Int2IP(_NetBitMask)
      End Get
   End Property

   Private ReadOnly Property _NetBitMask As Int32
      Get
         Return Not Length - 1
      End Get
   End Property

   ''' <summary>IP ohne Masken-Angabe</summary>
   Public Property Name As String
      Get
         Return Int2IP(Address)
      End Get
      Set(ByVal value As String)
         Address = IP2Int(value)
      End Set
   End Property

   ''' <summary>IP mit Maske, zB 168.230.16.3/24</summary>
   Public Property FullName As String
      Get
         Return Name & "/" & NetMaskLength
      End Get
      Set(ByVal value As String)
         Dim splits = value.Split("/"c)
         Name = (splits(0))
         NetMaskLength = If(splits.Length = 1, 32, Int32.Parse(splits(1)))
      End Set
   End Property

   ''' <summary>Untergrenze der SubnetMaske</summary>
   Public ReadOnly Property LBound As String
      Get
         Return Int2IP(LBoundAddress)
      End Get
   End Property
   ''' <summary>Obergrenze der SubnetMaske</summary>
   Public ReadOnly Property UBound As String
      Get
         Return Int2IP(LBoundAddress + Length - 1)
      End Get
   End Property
   ''' <summary>Untergrenze der SubnetMaske in Integer-Darstellung</summary>
   Public ReadOnly Property LBoundAddress As Integer
      Get
         Return Address And (Not Length - 1)
      End Get
   End Property
   ''' <summary>Größe des durch die SubnetMaske definierten Addressraumes</summary>
   Public ReadOnly Property Length As Integer
      Get
         Return 1 << (32 - NetMaskLength)
      End Get
   End Property

   ''' <summary>gibt die Integer-Darstellung einer IP zurück</summary>
   Public Shared Function IP2Int(ByVal ip As String) As Int32
      'split zerteilt ip in 4 strings, die je eine Zahl darstellen
      'Byte.Parse wandelt je ein Segment in ein Byte um
      'Select sammelt diese Bytes in ein IEnumerable(Of Bytes)
      'ToArray macht daraus ein Byte-Array
      Dim bytes = (From s In ip.Split("."c) Select Byte.Parse(s)).ToArray
      Array.Reverse(bytes)    'dreht die Reihenfolge um
      Return BitConverter.ToInt32(bytes, 0)
   End Function

   ''' <summary>bildet aussm Integer die String-Darstellung einer IP</summary>
   Public Shared Function Int2IP(ByVal n As Int32) As String
      Dim bytes = BitConverter.GetBytes(n)
      Array.Reverse(bytes)                'reihenfolge umdrehen
      'Byte-Array in ein String-Array konvertieren
      Dim strings = bytes.Select(Function(b) b.ToString)
      'Strings verketten und returnen
      Return String.Join(".", strings)
   End Function

End Class
